@@ -43,7 +43,8 @@ SEXP RGDAL_GetDriverLongName(SEXP sxpDriver);
 SEXP RGDAL_CloseHandle(SEXP sxpHandle);
 //SEXP RGDAL_DeleteHandle(SEXP sxpHandle);
 
-SEXP RGDAL_OpenDataset(SEXP filename, SEXP read_only, SEXP silent);
+SEXP RGDAL_OpenDataset(SEXP filename, SEXP read_only, SEXP silent,
+  SEXP allowedDr, SEXP options);
 SEXP RGDAL_CreateDataset(SEXP sxpDriver, SEXP sDim, SEXP sType,
   SEXP sOpts, SEXP sFile);
 SEXP RGDAL_GetDatasetDriver(SEXP sDataset);
